package App::tcpproxy;
BEGIN {
  $App::tcpproxy::AUTHORITY = 'cpan:GETTY';
}
# ABSTRACT: Simple TCP proxy for debugging connections
$App::tcpproxy::VERSION = '0.005';
use strict;
use warnings;



1;

__END__

=pod

=head1 NAME

App::tcpproxy - Simple TCP proxy for debugging connections

=head1 VERSION

version 0.005

=head1 DESCRIPTION

See L<tcpproxy.pl>.

=head1 SUPPORT

IRC

  Join #vonbienenstock on irc.freenode.net. Highlight Getty for fast reaction :).

Repository

  http://github.com/Getty/p5-app-tcpproxy
  Pull request and additional contributors are welcome

Issue Tracker

  http://github.com/Getty/p5-app-tcpproxy/issues

=head1 AUTHOR

Torsten Raudssus <torsten@raudss.us>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2014 by Torsten Raudssus.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
